/*
File: CBinomial.h
Purpose: Creates a base class CBinomial
which optimizes all the necessary likelihoods
Created: 10/19/05
Creator: Matt Wheeler
Modified: 10/07
Modifier: R. Woodrow Setzer
*/ 

#include "CMatrix.h"
#ifndef CBINOMIAL_CLASS
#define CBINOMIAL_CLASS



enum {basic, quantal_linear, quantal_quadratic, weibull, 
      gamma, multistage, logistic, log_logistic, probit, log_probit};
class CBinomial
{
 /////////////////////////////////////////////////////////////////////////
 // these friend functions are required for the minimization algorithms
 /////////////////////////////////////////////////////////////////////////
 friend void binomial_g(long int *nvar, double *x, long int *nf, double *g,
	                    long int *uiparm, double *urparm, void (*ufparm) ());
 friend void binomial_lk(long int *nvar,  double *x, long int *nf, double *f,
	             long int *uiparm,double *urparm, void (*ufparm)());
 //////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////
 public:
 // default constructor and destructor
        CBinomial::CBinomial(); 
        CBinomial::~CBinomial(); 
        // copy constructor
        CBinomial::CBinomial(const CBinomial &b);
 // Maximizes the likelihood
        virtual void max_likelihod(int iter,double gtol,double relConv); 
 // Sets the data in the likelihood
        void setData(double *,int row, int col); 
        void setData(CMatrix c); 
 // Sets up the lower and upper bounds 
 // for the likelihood
        void setUpperBounds(const double *); 
        void setLowerBounds(const double *); 
        void getUpperBounds(double *); 
        void getLowerBounds(double*); 
        void set_numParms(int numberParms);
 // if there is an error error_description computes it
        const char * error_description(); 
 // log likelihood function        
        double log_likelihood(const double *d);
 // gradient function
        void gradient(const double *v, double *g);
        double Neg2LogLike(){ return 2*logLikelihood;}
        double numParms(){
              return nparms;
        }
        double numNBParms(){
              int retVal = 0; 
              for (int i = 0; i < nparms; i++){
                  if(upper_bounds[i] > model_parameters[i] &&
                     lower_bounds[i] < model_parameters[i]){
                           retVal += 1; 
                  }
              }
              if (retVal == 0){
                 retVal = 1; // make sure there is at least one parameter
              }
              return (double)retVal;
        }
        
        CMatrix returnData(){ return data;}
        double predict(double dose){
               return link_function(model_parameters,dose);                
        }; 
        
        virtual CBinomial & operator = (CBinomial &b); 
        virtual void print_output(); 
        CMatrix varMatrix(); 
	// returns the number of parameters
	int get_numParms();
	// copy model parameters to external array
	// void copy_Parms(double v[][60], int i, int j);
	void copy_Parms(double *v);
     
protected:
        virtual double link_function(const double *v,double dose); // link function
        virtual void initializeParameters(){
                model_parameters[0]=0.5;
        }
                 
        double *model_parameters; // Model parameters
        double *upper_bounds; 
        double *lower_bounds; 
        
        CMatrix data;  
        
        int    error_status; // current error status
        int    nparms;       // number of parameters
        double logLikelihood;  // current value of the logLikelihood
        bool   isData; // flag specified if there is presently data in the 
                       // data matrix
        int class_type; 
        bool hasBeenMaximized; 
        
};    

double ComputeBIC(CBinomial *like);
double ComputeAIC(CBinomial *like); 
double ComputeBICM(CBinomial *like);
double ComputeAICM(CBinomial *like);
#endif    
