/*File: CMatrix.cpp
Author: Matt Wheeler
Created: 4/22/2005
Last Modified 6/16/2005
Last Modified By: Matt Wheeler

File Purpose: Implementation for the class CMatrix
defined in the file CMatrix.h. All functions reside in this file
save a few constructor classes which reside in the file Cmatrix.h.
*/ 

#include "CMatrix.h"

//Function: bool CMatrix::identity(int row, int col)
//Purpose: Sets the matrix to an identity matrix 
//of size (row,col)
bool CMatrix::identity(int row, int col){
     bool returnVal = false; 
     //delete the present matrix if it exists
     if (int_mat != NULL){
          gsl_matrix_free(int_mat);            
     } 
     //allocate the memory for the matrix and 
     //set it to the identity matrix
     int_mat = gsl_matrix_alloc(row,col);
     if (int_mat != NULL){
          gsl_matrix_set_identity(int_mat);
          returnVal = true; 
     }
     return returnVal;   
}; 
//Function: bool CMatrix::identity(int row, int col)
//Purpose: Sets the matrix to a square identity matrix 
//of size (sqr,sqr)
bool CMatrix::identity(int sqr){
     bool returnVal = false; 
     //delete the present matrix if it exists
     if (int_mat != NULL){
           gsl_matrix_free(int_mat);            
     } 
     //allocate the memory for the matrix and 
     //set it to the identity matrix
     int_mat = gsl_matrix_alloc(sqr,sqr);
     if (int_mat != NULL){
          gsl_matrix_set_identity(int_mat);
          returnVal = true; 
     }
     return returnVal;   
};      
//Function: bool CMatrix::zero(int row, int col)
//Purpose: Sets the matrix to all zeroes this matrix
//is of size (row,col)
bool CMatrix::zero(int row, int col){
     bool returnVal = false; 
     //delete the present matrix if it exists
     if (int_mat != NULL){
           gsl_matrix_free(int_mat);            
     } 
     //allocate the memory for the matrix and 
     //set it to the identity matrix
     int_mat = gsl_matrix_alloc(row,col);
     if (int_mat != NULL){
          gsl_matrix_set_zero(int_mat);
          returnVal = true; 
     }
     return returnVal;   
};     
//Function: bool CMatrix::zero(int sqr)
//Purpose: Sets the matrix to all zeroes this matrix
//is square (sqr,sqr)
bool CMatrix::zero(int sqr){
     bool returnVal = false; 
     //delete the present matrix if it exists
     if (int_mat != NULL){
            gsl_matrix_free(int_mat);            
     } 
     //allocate the memory for the matrix and 
     //set it to the identity matrix
     int_mat = gsl_matrix_alloc(sqr,sqr);
     if (int_mat != NULL){
          gsl_matrix_set_zero(int_mat);
          returnVal = true; 
     }
     return returnVal;   
};    
//Function: bool CMatrix::zero(double value, int row, int col)
//Purpose: Sets the matrix to a constant value specified by the
//double value. This matrix is of the dimensions (row,col)
bool CMatrix::constmatrix(double value, int row, int col){
     bool returnVal = false; 
     //delete the present matrix if it exists
     if (int_mat != NULL){
           gsl_matrix_free(int_mat);            
     } 
     //allocate the memory for the matrix and 
     //set it to the identity matrix
     int_mat = gsl_matrix_alloc(row,col);
     if (int_mat != NULL){
          gsl_matrix_set_all(int_mat,value);
          returnVal = true; 
     }
     return returnVal;   
};     

//Function: bool CMatrix::zero(double value, int row, int col)
//Purpose: Sets the matrix to a constant value specified by the
//double value. This matrix is square(sqr,sqr)
bool CMatrix::constmatrix(double value, int sqr){
     bool returnVal = false; 
     
     //delete the present matrix if it exists
     if (int_mat != NULL){
          gsl_matrix_free(int_mat);            
     } 
     //allocate the memory for the matrix and 
     //set it to the identity matrix
     int_mat = gsl_matrix_alloc(sqr,sqr);
     if (int_mat != NULL){
          gsl_matrix_set_all(int_mat,value);
          returnVal = true; 
     }
     return returnVal;   
};  
// Operator +
// Defines the addition of two matricies through the operator +
CMatrix CMatrix::operator + (const CMatrix &a){
       CMatrix temp; 
       if (int_mat != NULL && a.int_mat != NULL){
            temp.int_mat = gsl_matrix_alloc(a.int_mat->size1,a.int_mat->size2); 
            if(temp.int_mat != NULL){               
                        gsl_matrix_memcpy(temp.int_mat,int_mat);
                        int result = gsl_matrix_add(temp.int_mat,a.int_mat);                  
            }
            
       }
       return temp;              
}
// Operator -
// Overloads the operator - defining the subtraction of two matricies
CMatrix CMatrix::operator - (const CMatrix &a){
       CMatrix temp; 
       if (int_mat != NULL && a.int_mat != NULL){
            temp.int_mat = gsl_matrix_alloc(a.int_mat->size1,a.int_mat->size2); 
            if(temp.int_mat != NULL){               
                        gsl_matrix_memcpy(temp.int_mat,int_mat);
                        int result = gsl_matrix_sub(temp.int_mat,a.int_mat);                  
            }
            
       }
       return temp;              
}
//Operator *
// Overloads the operator * for left scalar multiplication of a matrix;
CMatrix CMatrix::operator * (const double a){
        CMatrix temp; 
        temp.constmatrix(a,int_mat->size1,int_mat->size2);
        gsl_matrix_mul_elements(temp.int_mat,int_mat); 
        return temp;                
}; 

//Operator *
// Overloads the operator * for right scalar multiplication of a matrix;
CMatrix operator * ( const double a, CMatrix b){ 
        return b*a; 
}

//Operator * const CMatrix &a
//takes two matrices a and b and multiplies them 
// together. 
CMatrix CMatrix::operator * (const CMatrix &a){
        CMatrix temp; 
        if ( int_mat != NULL && a.int_mat != NULL){
           //check to see if they can be multiplied
           //i.e #COL in the first matrix equal to the 
           //number of rows in the second; 
           if (int_mat->size2 == a.int_mat->size1){
                              temp.zero(int_mat->size1,a.int_mat->size2);
                              gsl_blas_dgemm (CblasNoTrans, CblasNoTrans,
                                               1.0, int_mat, a.int_mat,0.0,
                                               temp.int_mat);
           }else{
                 temp.zero(1,1); 
                 temp.badMatrix = true;       
           } 
           
        }    
        return temp;   
}

//Function: InitFromArray(double array[], int row, int col)
// Purpose takes an array and initializes the matrix to that 
// array
// Returns: A matrix initialized to the values of array and of
// diminsion (row,col). The matrix is initialized in row order i.e.
// given an array x[] = {1,2,3,4} the corresponding matrix 
// will be [1 2]  and not [1 3]
//         [3 4]          [2 4]
bool CMatrix::InitFromArray(double array[],int row,int col){
    bool returnVal = false; 
    //make sure that we delete any memory that might be allocated  
    if (int_mat !=NULL){
           gsl_matrix_free(int_mat);      
    }
    
    int_mat = gsl_matrix_alloc(row,col);     
    if (int_mat !=0){
        gsl_matrix_view A = gsl_matrix_view_array(array, row, col);
        gsl_matrix_memcpy(int_mat, &A.matrix);   
        returnVal = true; 
    }
    
    return returnVal; 
};

//Function: CMatrix inverse(); 
//Calculates the the inverse of the present matrix; 
//The invers of the matrix is returned. 
CMatrix CMatrix::inverse(){
    CMatrix temp,temp2; 
    int signum; 
    if (int_mat != NULL){
        temp.zero(int_mat->size1); 
        temp2.zero(int_mat->size1); 
        //Compute the LU decomposition
        gsl_permutation * p = gsl_permutation_alloc(int_mat->size1);
        gsl_permutation_init(p);
        gsl_matrix_memcpy(temp.int_mat,int_mat); 
        gsl_linalg_LU_decomp(temp.int_mat,p,&signum);
        //Use the LU decomposition to compute a inverse for the origional
        //matrix            
        int status =  gsl_linalg_LU_invert(temp.int_mat,p,temp2.int_mat);
        if (status){
            temp.badMatrix = true;   
        }
        gsl_permutation_free (p);
    }
    
    return temp2; 
    
}; 

//Function: CMatrix transpose()
//Purpose: Transposes the the current matrix
//Return: A CMatrix class transposed
CMatrix CMatrix::transpose(){
         CMatrix temp; 
         temp.zero(int_mat->size2,int_mat->size1);
         
         for (int i = 0; i < int_mat->size2; i++){
            for (int j = 0; j < int_mat->size1; j++){
                   temp.set(i,j,this->get(j,i));
            }       
                
         }
         //printf("NRow:%d:NCol:%d\n",temp.nrow(),temp.ncol());
         return temp;
}

//Function: CMatrix::print()
//Prints the contents of the current matrix to the
//stdout stream. 
void CMatrix::print(){
      for (int i = 0; i < int_mat->size1; i++){
        for (int j = 0; j < int_mat->size2; j++){
            printf("%1.10f ", this->get(i,j));   
        }     
            printf("\n"); 
      }
} 
//Operator =
//Overloads the operator = to allow for one matrix a to be given the 
// value of the the current matrix. 
CMatrix & CMatrix::operator = (const CMatrix &a){
      badMatrix = false; 
      //make sure that we delete any memory that might be allocated  
      if (int_mat !=NULL){
            gsl_matrix_free(int_mat);      
      }
                            
      if (a.int_mat != NULL){
            
             int_mat = gsl_matrix_alloc(a.int_mat->size1,a.int_mat->size2);  
             gsl_matrix_memcpy(int_mat,a.int_mat);              
      }else {
             int_mat = NULL;                          
      }
}

//Function: CMatrix::get(int row, int col)  
//Purpose: Gets the elent in row -row and column col
// where row is defined to be (0,nrows -1) and 
// column is defined to be (0,ncols -1)
// if it is outside this bound then GSL_NAN is returned
double CMatrix::get(int row, int col){
    if (int_mat !=NULL && 
        row >= 0 && row < int_mat->size1 &&
        col >= 0 && col < int_mat->size2){
        return gsl_matrix_get(int_mat,row,col);
    }   else{
        return GSL_NAN;        
    }
} 
//Function: CMatrix::sett(int row, int col)  
//Purpose: Sets the element in row -row and column col
//where row is defined to be (0,nrows -1) and 
//column is defined to be    (0,ncols -1)
//If it is outside this bound then nothing is done
void CMatrix::set(int row, int col,double dval){
    if (int_mat !=NULL &&
        row >= 0 && row < int_mat->size1 &&
        col >= 0 && col < int_mat->size2){
        gsl_matrix_set(int_mat,row,col,dval);
    }   
} 

//CMatrix::nrow() 
// Returns the number of rows allocated in the matrix
int CMatrix::nrow(){
    return int_mat->size1; 
} 

//CMatrix::ncol() 
// Returns the number of colums allocated in the matrix
int CMatrix::ncol(){
    return int_mat->size2;    
}   

//CMatrix::save(FILE *stream)
//Purpose given a binary file stream function saves the CMatrix data
//to the given file.
//Returns true if the matrix was saved
//        false if there were any problems
//If the matrix is null then it writes out 0 col and 0 rows
//which is consistant
bool CMatrix::save(FILE *stream){
    //check to see if I am saving a NULL matrix
    //or if I am saving an actual matrix
    if (int_mat != NULL){
        fwrite(&int_mat->size1,sizeof(int_mat->size1),1,stream); 
        fwrite(&int_mat->size2,sizeof(int_mat->size2),1,stream); 
              
        if (gsl_matrix_fwrite(stream,int_mat)){
            return false; 
        }else{
            return true; 
        } 
    }else{
        fwrite(&int_mat->size1,sizeof(int_mat->size1),1,stream); 
        fwrite(&int_mat->size2,sizeof(int_mat->size2),1,stream); 
        return true;         
    }
} 

//CMatrix::load(FILE *stream)
//Purpose given a binary file stream function loads the CMatrix data
//from the given file.
//Returns true if the matrix was loaded
//        saved if there were any problems
//If the matrix is null then it writes out 0 col and 0 rows
//which is consistant
bool CMatrix::load(FILE *stream){
    //check to see if I am saving a NULL matrix
    //or if I am saving an actual matrix
    int col; 
    int row; 
    if (int_mat != NULL){
        gsl_matrix_free(int_mat);
        int_mat = NULL; 
        fread(&row,sizeof(row),1,stream); 
        fread(&col,sizeof(col),1,stream); 
        if (row == 0 && col == 0){
            int_mat = NULL; 
        }else{
            int_mat= gsl_matrix_alloc(row,col); 
            if (gsl_matrix_fread(stream,int_mat)){
                return false; 
            }else{
                return true; 
            }
        } 
    }else{
        fread(&row,sizeof(row),1,stream); 
        fread(&col,sizeof(col),1,stream); 
        
        int_mat= gsl_matrix_alloc(row,col); 
        if (row == 0 && col == 0){
            int_mat = NULL; 
        }else{
            int_mat= gsl_matrix_alloc(row,col); 
            if (gsl_matrix_fread(stream,int_mat)){
                return false; 
            }else{
                return true; 
            }
        }         
    }    
    return true; 
} 

void CMatrix::deleterow(int rowstart, int rowend){
     gsl_matrix *tempMat; 
      
     if (rowend-rowstart+1 >= int_mat->size1-1){
         //delete the matrix and create a new matrix
         // of diminsion (1,1) = 0
         gsl_matrix_free(int_mat);
         int_mat = gsl_matrix_alloc(1,1); 
         gsl_matrix_set(int_mat,0,0,0.0); 
     }else{
         if(rowend > int_mat->size1-1){
                   rowend = int_mat->size1-1;           
         }
         if(rowstart <= rowend){
              // if the row end is beyond the last row
              // we make it the last row 
              int nrow = int_mat->size1-(rowend-rowstart+1); 
              int ncol = int_mat->size2;
              
              tempMat = gsl_matrix_alloc(nrow,ncol); 
              for (int i = 0; i < int_mat->size1; i++){
                      for (int j = 0; j < int_mat->size2; j++){
                          if ( i <  rowstart){
                             gsl_matrix_set(tempMat,i,j,
                                            gsl_matrix_get(int_mat,i,j));     
                          }else if (i > rowend){
                             gsl_matrix_set(tempMat,i-(rowend-rowstart+1),j,
                                            gsl_matrix_get(int_mat,i,j));     
                          }
                      }
              }          
              gsl_matrix_free(int_mat); 
              int_mat = tempMat; 
         }
     }
} 

void CMatrix::deleterow(int row){
     deleterow(row,row);    
} 
void CMatrix::deletecol(int colstart, int colend){
     gsl_matrix *tempMat; 
     if (colend-colstart+1 >= int_mat->size2-1){
         //delete the matrix and create a new matrix
         // of diminsion (1,1) = 0
         gsl_matrix_free(int_mat);
         int_mat = gsl_matrix_alloc(1,1); 
         gsl_matrix_set(int_mat,0,0,0.0); 
     }else{
         // if the row end is beyond the last row
         // we make it the last row
         if(colend > int_mat->size2-1){
                   colend = int_mat->size1-1;           
         }
         if(colstart <= colend){
              int nrow = int_mat->size1; 
              int ncol = int_mat->size2-(colend-colstart+1);
              
              tempMat = gsl_matrix_alloc(nrow,ncol); 
              for (int i = 0; i < int_mat->size1; i++){
                      for (int j = 0; j < int_mat->size2; j++){
                          if ( j <  colstart){
                             gsl_matrix_set(tempMat,i,j,
                                            gsl_matrix_get(int_mat,i,j));     
                          }else if (j > colend){
                             gsl_matrix_set(tempMat,i,j-(colend-colstart+1),
                                            gsl_matrix_get(int_mat,i,j));     
                          }
                      }
              }          
              gsl_matrix_free(int_mat); 
              int_mat = tempMat; 
         }
     }
} 
void CMatrix::deletecol(int col){
     deletecol(col,col); 
}

void CMatrix::addrow(int rows){
    gsl_matrix *tempMat; 
    tempMat = gsl_matrix_alloc(int_mat->size1+rows,int_mat->size2); 
    gsl_matrix_set_zero(tempMat); 
    
    gsl_matrix_view veiw = gsl_matrix_submatrix(tempMat,0,0,int_mat->size1,
                                                            int_mat->size2);
    gsl_matrix_memcpy(&veiw.matrix,int_mat); 
    gsl_matrix_free(int_mat); 
    int_mat = tempMat; 
} 

void CMatrix::addcol(int col){
    gsl_matrix *tempMat; 
    tempMat = gsl_matrix_alloc(int_mat->size1,int_mat->size2+col); 
    gsl_matrix_set_zero(tempMat); 
    
    gsl_matrix_view veiw = gsl_matrix_submatrix(tempMat,0,0,int_mat->size1,
                                                            int_mat->size2);
    gsl_matrix_memcpy(&veiw.matrix,int_mat); 
    gsl_matrix_free(int_mat); 
    int_mat = tempMat;      
}





