/*File: CMatrix.h
Author: Matt Wheeler
Created: 4/22/2005
Last Modified 6/16/2005
Last Modified By: Matt Wheeler

File Purpose: Implementation for the class CMatrix
defined in the file CMatrix.h. All functions reside in this file
save a few constructor classes which reside in the file Cmatrix.h.
*/ 

#include <stdio.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_permutation.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_math.h>

#ifndef __CLASS__MATRIX
#define __CLASS__MATRIX



class CMatrix{
   public:
          //constructors
          CMatrix(){
                    
                    badMatrix = false; 
                    int_mat = NULL;
          }
          // copy constructor
          CMatrix(const CMatrix &a){
                    badMatrix = false;                         
                    if (a.int_mat != NULL){
                          int_mat = gsl_matrix_alloc(a.int_mat->size1,a.int_mat->size2);  
                          gsl_matrix_memcpy(int_mat,a.int_mat);              
                    }else {
                          int_mat = NULL;                          
                    }
                    
          }

          //initialization routines 
          //for the MATRIX class 
          bool identity(int row, int col); 
          bool identity(int sqr); 
          bool zero(int row, int col); 
          bool zero(int sqr);
          bool constmatrix(double value, int row, int col);
          bool constmatrix(double value, int sqr);
          bool InitFromArray(double array[],int row,int col);
          
          //Delete and Add Rows
          void deleterow(int rowstart, int rowend); 
          void deleterow(int row); 
          void deletecol(int colstart, int colend); 
          void deletecol(int col); 

          void addrow(int row); 
          void addcol(int col);
          
          //retrieval of information
          double get(int row, int col); 
          void set(int row, int col,double dval); 
          int nrow(); 
          int ncol(); 
          /////////////////////////////
          //Save and load data routines
          bool save(FILE *stream); 
          bool load(FILE *stream); 
          /////////////////////////////
          
          // Operators for the matrix class e.g. +, -, *,=, and inverse
          CMatrix inverse(); 
          CMatrix transpose(); 
          CMatrix CMatrix::operator + (const CMatrix &a); 
          CMatrix CMatrix::operator - (const CMatrix &a); 
          //Scalar multiplication
          CMatrix CMatrix::operator * (const double a); 
          //Matrix multiplication
          CMatrix CMatrix::operator * (const CMatrix &a);           
          
          CMatrix & operator = (const CMatrix &a);          
          // for testing purposes only
          void print(); 
          
          ~CMatrix(){
                     if (int_mat != NULL){
                        gsl_matrix_free(int_mat);            
                     }           
          }
          
   private: 
          gsl_matrix * int_mat; 
          bool badMatrix; 
      
            
}; 

CMatrix operator * ( const double a, CMatrix b); 

#endif
