/*
Last Modified: 6/12/2007  Made the lower bound of the log-probit be 1e-8 instead
                          of 0.  This increases stability in the estimation. 


*/
#include <cstdlib>
#include <time.h>
#include <iostream>
#include <fstream>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_cdf.h>
#include "CMatrix.h"
#include "CBinomial.h"
#include "CGamma.h"
#include "CWeibullDR.h"
#include "CMultistage.h"
#include "CLogLogit.h"
#include "CLogProbit.h"
#include "CProbit.h"
#include "CQlinearDR.h"
#include "CQQuadratic.h"
#include "CLogistic.h"
// Current Version
#define VERSION_NUMBER "MABMD VERSION 1.0\n"
// 
int GLOBAL_numIter = 1000;
double GLOBAL_relConv = 1e-8; 
double GLOBAL_gConv = 1e-8;

using namespace std;
extern const int NUM_DR_MODELS = 9;
extern const int ADDED_RISK = 1;
extern const int EXTRA_RISK = 2;
int randomSeed; 
double ComputeAICC(CBinomial *like);
double ComputeAIC(CBinomial *like);
double ComputeBIC(CBinomial *like);
double ComputeAICM(CBinomial *like);
double ComputeBICM(CBinomial *like);
double ComputeKIC(CBinomial *like);
double ComputeKICM(CBinomial *like);

double  lowerCB( double alpha, double mean, double boots[], double acc, int nboots); 
// Added by RWS 10/31/2007: BSparms is a matrix of the parameter values and weights
// from each bootstrapped fit.  The columns are:
// wt1 parm11 parm12 parm13 wt2 parm21 parm22 parm23 ...
// where the first index gives model, and the second index on parm gives the parameter
// in the model.
// maxBSparms gives the actual number of columns used.  The fixed dimension
// used here is for a maximum polynomial degree of 30.
void modelAverage(CMatrix dataM, const bool *incl, double *ubounds[], 
                  double *lbounds[], int degreePoly, int riskType,  
                  int n_boot, double *bmd, double *bmdl, 
                  double BMR, double (*IC)(CBinomial *),double *a, 
                  double *weights, CBinomial *cpyModels[],double *GF_test,
                  double *GF_P_value, double *BSparms[], int *maxBSparms);

int main(int argc, char *argv[])
{
    
   	
    fstream infile; 
    if (argc > 1){
             infile.open(argv[1],fstream::in); 
                if (infile.fail()){
                    printf("ERROR OPENING INPUT FILE: "); 
                    printf(argv[1]);
                    printf("\n");
                    return -1;                  
                }
    }else{
             infile.open("input.txt",fstream::in); 
             if (infile.fail()){
                    printf("ERROR OPENING FILE: input.txt\n"); 
                    return -1;                  
             }
    }
     
    infile >> GLOBAL_numIter >> GLOBAL_relConv >> GLOBAL_gConv; 
    
    double temp; 
    int degreePoly;     
    // input the inclusion response variables 
    
    char tempString[250]; 
    void * ICCriterion; 
    // Set up  the results matrix
    CQlinearDR         qlin;    
    CWeibullDR      weibull;    
    CQQuadraticDR     qquad;    
    CLogistic      logistic;    
    CProbit          probit;    
    CMultiStage      mstage;    
    CLogProbit      logProbit;  
    CLogLogistic    logLogistic;
    CGamma          gamma;  
    char dateStr[100];
    int curDate = time(NULL); 
    CBinomial *fit_models[] = {&qlin,&qquad,&mstage,&logistic,
                               &probit,&weibull,&logProbit,&logLogistic,
                               &gamma};   
    double modelWeights[NUM_DR_MODELS];
    double alpha = 0.05; 
    int nboots; 
                    
    CMatrix dataMatrix; 
    CMatrix variance; 
    
    
    double bmr; 
    
   
    double bic_bmd,bic_bmdl;
    double aic_bmd,aic_bmdl;
    fstream dinput;
 
    bool models[]= {true,true,true,true,true,true,true,true,true}; 
    char *model_names[] = {"BIC","AIC","KIC","BIC Non-Bounded Only", 
                            "AIC Non-Bounded Only", "KIC Non-Bounded Only"}; 

    
    // input the data response variables 
    for (int i = 0; i < 9; i ++){
        infile >> models[i];     
    }
    // input the seed for the random number generator
    infile >> randomSeed; 
    // if the random seed is zero then we get the current time
    if (randomSeed == 0){
          randomSeed = time(NULL);          
    }
    // read in the necessary BMD infomation
    int tempSeed = randomSeed; 
    int weight_type; 
    int risk_type; 
    int bsprint; 
    infile >> weight_type >> risk_type >> bmr >> alpha >> nboots >> bsprint; 
    alpha = 1-alpha; 
    infile >> degreePoly; 
    switch(weight_type){
         case 1: 
              ICCriterion = (void*)ComputeBIC; 
              break; 
         case 2: 
              ICCriterion = (void*)ComputeAIC; 
              break; 
         case 3: 
              ICCriterion = (void*)ComputeKIC; 
              break; 
         case 4: 
              ICCriterion = (void*)ComputeBICM; 
              break; 
         case 5: 
              ICCriterion = (void*)ComputeAICM; 
              break; 
         case 6: 
              ICCriterion = (void*)ComputeKICM; 
              break; 
         default: 
              ICCriterion = (void*)ComputeBIC; 
    }
              
    int dataLines; 
    infile >> dataLines; 
    
    double data[dataLines*3]; 
    // input the data
    for (int j = 0; j < dataLines; j++){
        infile >> data[j*3] >> data[j*3+1] >> data[j*3+2];     
    }
                         
    dataMatrix.InitFromArray(data,dataLines,3); 

    // set the bounds for the different models
    double  logProbitLBounds[] = {-18,1.0e-8,1e-8}; double  logProbitUBounds[] =  {18,18,1-1e-8};
    double  qlinLBounds[] = {0,0};        double  qlinUBounds[] = {1,18}; 
    double  qquadLBounds[] = {0,0};       double  qquadUBounds[] = {1,18}; 
    double  weibLBounds[] = {0,0,0.5};      double  weibUBounds[] = {1,18,18};
    double  logLBounds[]  = {-18,-18};    double  logUBounds[]  = {18,18}; 
    double  probitLBounds[]  = {-18,-18}; double   probitUBounds[]  = {18,18}; 
    double  gammaLBounds[]   =  {1.0,0,0};      double  gammaUBounds[] = {18,18,1};
    double  logLogitLBounds[] = {-18,0.5,0}; double  logLogitUBounds[] =  {18,18,1};
     
    // EPA Bounds for log-probit,log-logistic, Weibull, and Gamma
/*  double  logProbitLBounds[] = {-18,1.0,0}; double  logProbitUBounds[] =  {18,18,1};
    double  weibLBounds[] = {0,0,1.0};      double  weibUBounds[] = {1,18,18}; 
    double  gammaLBounds[]   =  {1.0,0,0};      double  gammaUBounds[] = {18,18,1};
    double  logLogitLBounds[] = {-18,1.0,0}; double  logLogitUBounds[] =  {18,18,1};*/
    
    double  multiLBounds[degreePoly]; 
    double  multiUBounds[degreePoly]; 
    multiLBounds[0] = 0; // lower bound for gamma term for the multistage model
    multiUBounds[0] = 1; // upper bound for gamma term for the multistage model 
        
    for (int i = 1; i < degreePoly+1; i++){
        multiLBounds[i] = 0; 
        multiUBounds[i] = 18;  
    }
    
    char model_label[]={"Model           "}; 
    char *labels[9] = {"Quantal Linear   ", 
                       "Quantal Quadratic",
                       "Multistage       ", 
                       "Logistic         ", 
                       "Probit           ", 
                       "Weibull          ",
                       "Log-Probit       ",
                       "Log-Logistic     ", 
                       "Gamma            "}; 
     
    double *lbounds[] = {qlinLBounds,qquadLBounds,multiLBounds,
                         logLBounds,probitLBounds,weibLBounds,logProbitLBounds,
                         logLogitLBounds,gammaLBounds}; 

    double *ubounds[] = {qlinUBounds,qquadUBounds,multiUBounds,
                         logUBounds,probitUBounds,weibUBounds,logProbitUBounds,
                         logLogitUBounds,gammaUBounds};
    ////////////////////////////////////////////////////////////////////////////
   

    
  
    double z0 = gsl_cdf_gaussian_Pinv(0.975,1);  
    double bmd; 
    double bmdl[nboots];
    
    //Change the method of allocation 1-3-2008
    //Instead of a huge flat array it allocates the memory in peices
    //This prevents crashes on larger bootstrap replications
    double *BSparms[nboots] ;
    for (int i = 0; i < nboots; i++){
        BSparms[i] = (double*)malloc(sizeof(double)*60); 
    }
    
    int maxBSparms;
    double acceleration; 
    double gftest,pvalue; 
    // dataMatrix.print(); 
    if ( risk_type == 1) {
        modelAverage(dataMatrix, models, ubounds, 
                     lbounds, degreePoly, ADDED_RISK,  
                     nboots,  &bmd, bmdl, 
                     bmr, (double(*)(CBinomial*))ICCriterion, &acceleration,
                     modelWeights,fit_models,&gftest,&pvalue,
		     BSparms,&maxBSparms); 
    } else {
            modelAverage(dataMatrix, models, ubounds, 
                     lbounds, degreePoly, EXTRA_RISK,  
                     nboots,  &bmd, bmdl, 
                     bmr, (double(*)(CBinomial*))ICCriterion, &acceleration,
			 modelWeights,fit_models,&gftest,&pvalue,
			 BSparms, &maxBSparms);    
    }
    double talpha = lowerCB( alpha, bmd, bmdl, acceleration, nboots);
    int i = (int)(nboots*(talpha) - 1) ;
    if ( i < 0) i = 0; // prevent underflow
      
    time_t rawtime;
    struct tm * timeinfo;
    time ( &rawtime );
    timeinfo = localtime ( &rawtime );
    printf("%s%s\n", VERSION_NUMBER, asctime(timeinfo));
    printf("This program's results are given \"as is\", without warranty, either expressed \nor implied by the National Institute for Occupational Safety and Health.\n\n\n"); 
    
    printf("INPUT DATA\n");
    printf("Dose \t         Count \t        Observed\n");
    
    printf("--------------------------------------------------------------------------\n");
    for(int i = 0; i < dataMatrix.nrow(); i++){
        printf("%f\t%8.0f\t%8.0f\n",dataMatrix.get(i,0),dataMatrix.get(i,1),dataMatrix.get(i,2));            
    }
    printf("--------------------------------------------------------------------------\n\n\n");
    printf("Model Fit Statistics\n");
    printf("--------------------------------------------------------------------------\n");
    printf("%s\t%s\t-2log(L)\t    AIC\t  \t   BIC\n",model_label,"Weight"); 
    printf("--------------------------------------------------------------------------\n");
    for (int i = 0; i < NUM_DR_MODELS; i++){
        if (models[i]){
                 printf("%s\t%1.3f\t%8.2f\t%8.2f\t%8.2f\n",labels[i],modelWeights[i],
                                    fit_models[i]->Neg2LogLike(), 
                                    ComputeAIC(fit_models[i]),
                                    ComputeBIC(fit_models[i]));
        }
    }   
     
    printf("--------------------------------------------------------------------------\n");
    printf("\n\n\n"); 
    printf("------------------------------------------\n");
    printf("'Average-Model' Benchmark Dose Estimate   \n"); 
    printf("------------------------------------------\n");
    printf("Nominally Specified Confidence Level:%1.3f\n",1-alpha); 
    printf("Weighting Criterion: %s\n",model_names[weight_type-1]); 
    if (risk_type == 1){
       printf("BMD Calculation: Added Risk\n"); 
    }else{
       printf("BMD Calculation: Extra Risk\n");           
    }
    printf("BMR: %f\n",bmr); 
    printf("BMD: %1.12f\nBMDL(BCa):%1.12f\nBMDL(Percentile):%1.12f\n",bmd,bmdl[i],bmdl[(int)(nboots*alpha)-1]);
    printf("Acceleration: %f\n",acceleration); 
    printf("Bootstrap Resamples: %d\n", nboots); 
    printf("Random Seed: %d\n",tempSeed); 
    printf("------------------------------------------\n\n\n"); 
    printf("------------------------------------------\n");
    printf("'Average-Model' Goodness of Fit Test\n"); 
    printf("------------------------------------------\n");
    printf("Test Statistic   : %f\n", gftest); 
    printf("Bootstrap P-Value: %f\n", pvalue); 
    printf("------------------------------------------\n\n\n"); 
    
    CMatrix bob; 
    
    for (int i = 0; i < NUM_DR_MODELS; i++){
        if(models[i]){
             fit_models[i]->print_output();  
             bob= fit_models[i]->varMatrix(); 
         
             printf("------------------------------------------\n");  
          printf("\n");
        }          
    }
  
    if (bsprint){
        printf("Bootstrap Resamples\n"); 
        printf("------------------------------------------\n");  
        for (int i = 0; i < nboots; i++){
	      printf("%.14f",bmdl[i]);
	     // remove the bsprint for the MadrBMD
         /* if (bsprint == 2){
      	     for (int j = 0; j < maxBSparms; j++){
	             printf(" %.14f", BSparms[i][j]); 
            }
          }*/  
          
	  printf("\n");
      }
    } 
    infile.close(); 
      
    return EXIT_SUCCESS;
}

double lowerCB( double alpha, double mean, double boots[], double acc, int nboots){
       double counts = 0; 
       double p = 0; 
       double z0 = 0; 
       
       double z_alpha = gsl_cdf_gaussian_Pinv(alpha,1); 
       for (int i = 0; i < nboots; i++){
           if (boots[i] <= mean) counts+=1; 
       }
       counts = counts/nboots;
       
       z0 = gsl_cdf_gaussian_Pinv(counts,1);     
       double      x = gsl_cdf_gaussian_P(z0+(z0+z_alpha)/(1-acc*(z0+z_alpha)),1);
        
       return gsl_cdf_gaussian_P(z0+(z0+z_alpha)/(1-acc*(z0+z_alpha)),1);
}
