/***************************************************************
 * run_dmngb -- sets up auxiliary arrays to run dmngb, and tries
 * repeatedly (by tweaking the initial parameter values) to find
 * an optimum.
 * input:
 *       nparm: the number of parameters to be estimated
 *       start: array of initial values
 *       lower: array of lower bounds
 *       upper: array of upper bounds
 *       Dscale: array of scales for parameters
 *       maxloglik: -log-likelihood at full model (for absolute function
 *                  convergence)
 *       relconv: relative function convergence limit
 *       parmconv: parameter convergence limit
 *       itmax: maximum number of iterations allowed dmngb
 *       trymax: maximum number of retries allowed when dmngb does not
 *               converge
 *       func: function to minimize
 *       grad: gradient of func
 *       uiparm: array of user-specified long int values for func and grad
 *       urparm: array of user-specified double values for func and grad
 *       ufparm: user specified function argument for func and grad
 *       debug: if 0, no debugging output; if 1, debugging output to unit 16
 * output:
 *       start: final parameter vector returned by dmngb
 *       fval: value of the function at the optimum
 * return value:
 *      -2: unable to allocate required memory
 *      -1: problem too big (MAXPARMS exceeded)
 *       0: for any of the values returned by dmngb that indicate convergence
 *          (including singular convergence): iv[0] in 1..7.
 *       other: the value returned in iv[0] on return from dmngb the last try.
 ******************************************************************/

extern "C" int run_dmngb(int nparm, double start[], double lower[], double upper[],
	                     double maxloglik,
                      	 double relconv, double parmconv, int itmax, int trymax,
                         void (*func)(), void (*grad)(),
                         long int *uiparm, double *urparm, void (*ufparm)(),
                         int debug, double *fval);
